<?php

namespace Drupal\wordfilter\Entity;

use \Drupal\Core\Config\Entity\ConfigEntityBase;
use \Drupal\wordfilter\WordfilterItem;
use \Drupal\wordfilter\Plugin\WordfilterProcessInterface;

/**
 * Defines the Wordfilter configuration entity.
 *
 * @ConfigEntityType(
 *   id = "wordfilter_configuration",
 *   label = @Translation("Wordfilter configuration"),
 *   admin_permission = "administer wordfilter configurations",
 *   handlers = {
 *     "list_builder" = "Drupal\wordfilter\WordfilterConfigurationListBuilder",
 *     "form" = {
 *       "add" = "Drupal\wordfilter\Form\WordfilterConfigurationForm",
 *       "edit" = "Drupal\wordfilter\Form\WordfilterConfigurationForm",
 *       "delete" = "Drupal\wordfilter\Form\WordfilterConfigurationDeleteForm"
 *     },
 *     "route_provider" = {
 *       "html" = "Drupal\wordfilter\WordfilterConfigurationHtmlRouteProvider",
 *     },
 *     "access" = "Drupal\wordfilter\WordfilterConfigurationAccessControlHandler",
 *   },
 *   config_prefix = "wordfilter_configuration",
 *   entity_keys = {
 *     "id" = "id",
 *     "label" = "label",
 *     "uuid" = "uuid",
 *   },
 *   config_export = {
 *     "id",
 *     "label",
 *     "uuid",
 *     "process_id",
 *     "items",
 *   },
 *   links = {
 *     "canonical" = "/admin/config/wordfilter_configuration/{wordfilter_configuration}",
 *     "add-form" = "/admin/config/wordfilter_configuration/add",
 *     "edit-form" = "/admin/config/wordfilter_configuration/{wordfilter_configuration}/edit",
 *     "delete-form" = "/admin/config/wordfilter_configuration/{wordfilter_configuration}/delete",
 *     "collection" = "/admin/config/wordfilter_configuration"
 *   }
 * )
 */
class WordfilterConfiguration extends ConfigEntityBase implements WordfilterConfigurationInterface {

  /**
   * The Wordfilter configuration ID.
   *
   * @var string
   */
  protected $id;

  /**
   * The Wordfilter configuration label.
   *
   * @var string
   */
  protected $label;

  /**
   * The assigned Wordfilter process id.
   * 
   * @var string
   */
  protected $process_id = 'default';
  
  /**
   * The assigned Wordfilter process object.
   * 
   * @see ::getProcess().
   * 
   * @var \Drupal\wordfilter\Plugin\WordfilterProcessInterface 
   */
   protected $process = NULL;

  /**
   * The filtering items as arrays.
   *
   * @var array
   */
  protected $items = [['delta' => 0, 'substitute' => '', 'filter_words' => '']];

  /**
   * {@inheritdoc}
   */
  public function getProcess() {
    if (isset($this->process) && ($this->process->getPluginId() == $this->process_id)) {
      return $this->process;
    }

    /**
     * @var \Drupal\wordfilter\Plugin\WordfilterProcessManager
     */
    $manager = \Drupal::service('plugin.manager.wordfilter_process');
    $this->setProcess($manager->createInstance($this->process_id));
    return $this->getProcess();
  }

  /**
   * {@inheritdoc} 
   */
  public function setProcess(WordfilterProcessInterface $process) {
    $this->set('process_id', $process->getPluginId());
    $this->process = $process;
  }

  /**
   * {@inheritdoc}
   */
  public function getItems() {
    $item_objects = [];
    foreach ($this->items as &$item) {
      $item_objects[$item['delta']] = new WordfilterItem($this, $item);
    }
    return $item_objects;
  }

  /**
   * {@inheritdoc}
   */
  public function newItem($delta = NULL) {
    if (!isset($delta)) {
      $delta = 0;
      foreach ($this->items as $item) {
        if ($delta <= $item['delta']) {
          $delta = $item['delta'] + 1;
        }
      }
    }
    $this->items[] = ['delta' => $delta, 'substitute' => '', 'filter_words' => ''];
    $item_objects = $this->getItems();
    return $item_objects[$delta];
  }

  /**
   * {@inheritdoc}
   */
  public function removeItem(WordfilterItem $item_object) {
    if ($item_object->getParent() !== $this) {
      return FALSE;
    }
    foreach ($this->items as $key => $item) {
      if ($item['delta'] === $item_object->getDelta()) {
        unset($this->items[$key]);
        if (!count($this->items)) {
          // Fallback: At least one empty item must be present.
          $this->items = [['delta' => 0, 'substitute' => '', 'filter_words' => '']];
        }
        return TRUE;
      }
    }
    return FALSE;
  }
}
